//******************************************************************************
//Name          : STFT.cs
//Description   : Implementation to experience the beauty and the pitfalls of STFT
//Author        : Dr. Oliver Kluge, 90411 Nuernberg, Germany
//Copyright(C)  : Oliver Kluge 2014 
//------------------------------------------------------------------------------
//******************************************************************************

using System;
using Display;

class Spectrum
{
    const int samplerate = 256;                        // Abtastwerte im Abtastintervall
    const double PI      =   3.141592653589793;

    public double[] signal = new double[samplerate];    // Signalverlauf
    public double[] arg = new double[samplerate];       // x-axis <=> k
    public double[] arg1 = new double[samplerate / 2];  // x-axis <=> f
    public double[] arg2;                               // y-axis <=> k
    public double[,] KS;                                // Kurzzeitspektrum
    private double[] xw = new double[samplerate];       // Signal + Fensterung


    public int Samplerate
    {
        get { return samplerate; }
    }

    public void CreateSignal(int select)
    {
        // FreqChange
        if (select == 1)
        { 
            for (int k=0; k<samplerate; k++)
            {
                if (k < samplerate/2)
                {
                    signal[k] = Math.Sin(2*PI*52*k/samplerate);
                }
                else if (k >= 3*samplerate/4 && k < (3*samplerate/4 + samplerate/16))
                {
                    signal[k] = Math.Sin(2*PI*23*k/samplerate);
                }
            }
        }

        // Sinus
        if (select == 2)
        {
            for (int k = 0; k < samplerate; k++)
            {
                signal[k] = Math.Sin(2 * PI * 15 * k / samplerate);
            }
        }

        // SwitchingTransient
        if (select == 3)
        {
            double tau = 0;

            for (int k = 0; k < samplerate; k++)
            {
                signal[k] = Math.Sin(2*PI*16.7*k/samplerate);

                if (k >= 2*samplerate/4)
                {
                    signal[k] += Math.Exp(-tau/80.0) * Math.Sin(PI/2.0 + 2*PI*123*k/samplerate);
                }

                tau += 0.3;
            }
        }

        // store argument data
        for (int k = 0; k < samplerate; k++)
        {
            arg[k] = k;     // timeindex k
        }
        for (int k = 0; k < samplerate/2; k++)
        {
            arg1[k] = k;    // frequency
        }
    }

    public void Hanning(int Start, int M)
    {
        for (int k=Start; k<Start+M; k++)
        {
            xw[k] = signal[k]*(0.5-0.5*Math.Cos(2*PI*k/M));
        }
    }

    public void STFT(double[] x, int intervals)
    {
        double real = 0;
        double imag = 0;

        KS = new double[intervals, samplerate/2];
        arg2 = new double[intervals];

        // loop over all the subintervals
        for (int subinterval=0; subinterval<intervals; subinterval++)
        {
            // use window function x[] -> xw[]
            //Hanning((samplerate/intervals)*subinterval, samplerate/intervals);

            // loop over all the frequencies
            for (int freq=0; freq<samplerate/2; freq++)
            {
                real = 0;
                imag = 0;

                // calculate DFT-kernel (without overlaping of the subintervals)
                for (int k=(samplerate/intervals)*subinterval; k<(samplerate/intervals)*(1+subinterval); k++)
                {
                    real += x[k] * Math.Cos(2*PI*freq*k/samplerate);
                    imag += x[k] * Math.Sin(2*PI*freq*k/samplerate);
                }
                real *= 2.0/samplerate;
                imag *= 2.0/samplerate;

                KS[subinterval, freq] = Math.Sqrt(real*real + imag*imag);
            }

            KS[subinterval, 0] /= 2.0;                              // correction offset
            arg2[subinterval] = (subinterval*samplerate)/intervals; // timeindex k
        }
    }
}


class MainApp
{
    [STAThread]
    static void Main()
    {
        string userinput = "";
        int select1 = 0;
        int select2 = 0;

        while (select1 <= 3)
        {
            Spectrum mySpectrumObj = new Spectrum();

            Console.Write("Bitte Signal auswhlen:\n\n");
            Console.Write("FreqChange          -> 1\n");
            Console.Write("Sinus               -> 2\n");
            Console.Write("Switching transient -> 3\n");
            Console.WriteLine();
            userinput = Console.ReadLine();
            select1 = Convert.ToInt32(userinput);

            mySpectrumObj.CreateSignal(select1);

            Console.WriteLine();
            Console.WriteLine();
            Console.Write("Fensterweite:\n\n");
            Console.Write("samplerate/2  ->  2\n");
            Console.Write("samplerate/4  ->  4\n");
            Console.Write("samplerate/8  ->  8\n");
            Console.Write("samplerate/16 -> 16\n");
            Console.Write("samplerate/32 -> 32\n");
            Console.WriteLine();
            userinput = Console.ReadLine();
            select2 = Convert.ToInt32(userinput);

            mySpectrumObj.STFT(mySpectrumObj.signal, select2);

            // create a chart for displaying that data...
            DisplayChart mydisplay1 = new DisplayChart(mySpectrumObj.arg, mySpectrumObj.signal, "Time domain", "k", "x(k)");
            mydisplay1.Plot();

            DisplayChart mydisplay2 = new DisplayChart(mySpectrumObj.arg1, mySpectrumObj.arg2, mySpectrumObj.KS, "Spectrogram", "f", "k");
            //mydisplay2.Plot_3d();
            mydisplay2.Heatmap();

            for (int i = 0; i < 20; i++) Console.WriteLine();
        }
    }
}
